import json
import logging
from datetime import datetime
from typing import Dict, List, Optional, Union

from _qwak_proto.qwak.offline.serving.v1.feature_values_pb2 import (
    FeaturesetFeatures as ProtoFeaturesetFeatures,
)
from _qwak_proto.qwak.offline.serving.v1.offline_serving_async_service_pb2 import (
    FeatureValuesRequestStatus as ProtoFeatureValuesRequestStatus,
    FileFormat as ProtoFileFormat,
    GetFeatureValuesResultResponse as ProtoGetFeatureValuesResultResponse,
)
from _qwak_proto.qwak.offline.serving.v1.options_pb2 import (
    OfflineServingQueryOptions,
    OfflineServingQueryOptions as ProtoOfflineServingQueryOptions,
)
from _qwak_proto.qwak.offline.serving.v1.population_pb2 import (
    ArrowSchemaJson as ProtoArrowSchemaJson,
    Population as ProtoPopulation,
    PopulationFile as ProtoPopulationFile,
    TimedPopulation as ProtoTimedPopulation,
)
from google.protobuf.timestamp_pb2 import Timestamp as ProtoTimestamp
from qwak.clients.feature_store.offline_serving_client import FsOfflineServingClient
from qwak.exceptions import QwakException
from qwak.feature_store._common.packaging import upload_to_s3
from qwak.feature_store._common.value import (
    UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG,
)
from qwak.feature_store.offline._offline_serving_validations import (
    validate_features,
    validate_point_in_time_column_in_population,
)
from qwak.feature_store.offline.feature_set_features import FeatureSetFeatures
from qwak.utils.datetime_utils import datetime_to_pts

try:
    import pandas as pd
except ImportError:
    pass


class OfflineClientV2:
    """
    A class used to retrieve data from the offline store - mainly used to get train data for models.
    It requires qwak configure and aws access.
    """

    POPULATION_CONTENT_TYPE = "application/octet-stream"

    def __init__(self):
        self._fs_offline_serving_client = FsOfflineServingClient()

    def get_feature_range_values(
        self,
        features: FeatureSetFeatures,
        start_date: datetime,
        end_date: datetime,
        population: Optional["pd.DataFrame"] = None,
        include_featureset_version_column: bool = False,
    ) -> "pd.DataFrame":
        """
        :param features: a list of features to fetch
        :param start_date: lower bound of the range to fetch
        :param end_date: upper bound of the range to fetch
        :param population: a pandas data-frame with a column for each entity key of the feature-set defined in `features`
                           argument. if defenind- result will be filtered by the given entity values.
        :param include_featureset_version_column: whether to include featureset version column in resulting dataframe

        :return: a pandas dataframe - all feature values for
         all entites under the given date range

        each row in the returned data-frame is constructed by retrieving the requested features of the entity
         key(s) for all entity values in within the defined date tange.

        Examples:
        >>> from datetime import datetime
        >>> from qwak.feature_store.offline import OfflineClientV2
        >>> from qwak.feature_store.offline.feature_set_features import FeatureSetFeatures
        >>>
        >>> start_date = datetime(year=2021, month=1, day=1)
        >>> end_date = datetime(year=2021, month=1, day=3)
        >>> features = FeatureSetFeatures(
        >>>     feature_set_name='user_purchases',
        >>>     feature_names=['number_of_purchases', 'avg_purchase_amount']
        >>> )
        >>> offline_feature_store = OfflineClientV2()
        >>>
        >>> train_df = offline_feature_store.get_feature_range_values(
        >>>                features=features,
        >>>                start_date=start_date,
        >>>                end_date=end_date)
        >>>
        >>> print(train_df.head())
        >>> #	     uuid	         timestamp	      user_purchases.number_of_purchases	user_purchases.avg_purchase_amount
        >>> # 0	      1	        2021-01-02 17:00:00	                 76	                                4.796842
        >>> # 1	      1	        2021-01-01 12:00:00	                 5	                                1.548000
        >>> # 2	      2	        2021-01-02 12:00:00	                 5	                                5.548000
        >>> # 3	      2	        2021-01-01 18:00:00	                 5	                                2.788000
        """
        try:
            import pandas as pd
        except ImportError:
            raise QwakException(
                "Missing 'pandas' dependency required for fetching data from the offline store."
                f" {UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG}"
            )

        validate_features(features)
        population_proto: Optional[ProtoPopulation] = None
        if population is not None:
            population_proto = self._prepare_and_get_population(population)

        options: OfflineServingQueryOptions = ProtoOfflineServingQueryOptions(
            include_featureset_version_column=include_featureset_version_column
        )

        features_proto: ProtoFeaturesetFeatures = features.to_proto()
        try:
            logging.info("Executing Feature Values in range query")

            lower_time_bound_proto: ProtoTimestamp = datetime_to_pts(start_date)
            upper_time_bound_proto: ProtoTimestamp = datetime_to_pts(end_date)
            response: ProtoGetFeatureValuesResultResponse = (
                self._fs_offline_serving_client.get_feature_values_in_range_blocking(
                    features=features_proto,
                    lower_time_bound=lower_time_bound_proto,
                    upper_time_bound=upper_time_bound_proto,
                    result_file_format=ProtoFileFormat.FILE_FORMAT_CSV,
                    population=population_proto,
                    options=options,
                )
            )

            res: pd.DataFrame = OfflineClientV2._results_response_to_df(response)
            return res
        except QwakException as qwak_exception:
            raise QwakException(
                f"Got the following Qwak generated exception: {qwak_exception}"
            )
        except Exception as e:
            raise QwakException(f"Got the following run-time exception: {e}")

    def get_feature_values(
        self,
        features: List[FeatureSetFeatures],
        population: "pd.DataFrame",
        point_in_time_column_name: str = "timestamp",
        include_featureset_version_column: bool = False,
    ) -> "pd.DataFrame":
        """
        :param features: a list of FeatureSetFeatures to fetch.
        :param population: a pandas data-frame with a point in time column
                           and a column for each entity key of the feature-sets defined in `features`.
        :param point_in_time_column_name: the column name of the point in time column (default - timestamp)
        :param include_featureset_version_column: whether to include featureset version column in resulting dataframe

        :return: a pandas data-frame - the population joined with the feature values for all
                                       the requested entities and features.

        each row in the returned data-frame is constructed by retrieving the requested features of the entity key(s) for
        the specific entity value(s) in the population and on the specific point in time defined.

        Feature sets should be named [Feature Set Name].[Feature Name],
        i.e: user_purchases.number_of_purchases.

        Examples:
        >>> import pandas as pd
        >>> from qwak.feature_store.offline import OfflineClientV2
        >>> from qwak.feature_store.offline.feature_set_features import FeatureSetFeatures
        >>>
        >>> population_df = pd.DataFrame(
        >>>     columns= ['uuid',       'timestamp'     ],
        >>>     data   =[[ '1'  , '2021-01-02 17:00:00' ],
        >>>              [ '2'  , '2021-01-01 12:00:00' ]])
        >>>
        >>> features = [
        >>>     FeatureSetFeatures(feature_set_name='user_purchases',
        >>>                        feature_names=['number_of_purchases', 'avg_purchase_amount']),
        >>> ]
        >>> offline_feature_store = OfflineClientV2()
        >>>
        >>> train_df = offline_feature_store.get_feature_values(
        >>>                features=features,
        >>>                population=population_df,
        >>>                point_in_time_column_name='timestamp')
        >>>
        >>> print(train_df.head())
        >>> #	     uuid	         timestamp	      user_purchases.number_of_purchases	user_purchases.avg_purchase_amount
        >>> # 0	      1	        2021-04-24 17:00:00	                 76	                                4.796842
        >>> # 1	      2	        2021-04-24 12:00:00	                 5	                                1.548000
        """
        try:
            import pandas as pd
        except ImportError:
            raise QwakException(
                "Missing 'pandas' dependency required for fetching data from the offline store."
                f" {UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG}"
            )

        [validate_features(feature) for feature in features]
        validate_point_in_time_column_in_population(
            population=population, point_in_time_column_name=point_in_time_column_name
        )
        population_proto: ProtoPopulation = self._prepare_and_get_population(population)
        timed_population_proto: ProtoTimedPopulation = ProtoTimedPopulation(
            timestamp_column_name=point_in_time_column_name, population=population_proto
        )
        options: OfflineServingQueryOptions = ProtoOfflineServingQueryOptions(
            include_featureset_version_column=include_featureset_version_column
        )

        features_proto: List[ProtoFeaturesetFeatures] = [f.to_proto() for f in features]
        try:
            logging.info("Executing Feature Values query")
            response: ProtoGetFeatureValuesResultResponse = (
                self._fs_offline_serving_client.get_feature_values_blocking(
                    features=features_proto,
                    result_file_format=ProtoFileFormat.FILE_FORMAT_CSV,
                    population=timed_population_proto,
                    options=options,
                )
            )

            res: pd.DataFrame = OfflineClientV2._results_response_to_df(response)
            return res
        except QwakException as qwak_exception:
            raise QwakException(
                f"Got the following Qwak generated exception: {qwak_exception}"
            )
        except Exception as e:
            raise QwakException(f"Got the following run-time exception: {e}")

    def _prepare_and_get_population(
        self,
        popuation_df: "pd.DataFrame",
    ) -> ProtoPopulation:
        population_arrow_schema_json = OfflineClientV2._pandas_to_json_arrow_schema(
            popuation_df
        )
        population_upload_file_url = (
            self._fs_offline_serving_client.get_population_file_upload_url()
        )
        population_stream = popuation_df.to_parquet(index=False, coerce_timestamps="ms")

        upload_to_s3(
            population_upload_file_url, population_stream, self.POPULATION_CONTENT_TYPE
        )

        return ProtoPopulation(
            population_file=ProtoPopulationFile(
                file_url=population_upload_file_url,
                arrow_schema_json=ProtoArrowSchemaJson(
                    schema=population_arrow_schema_json
                ),
            )
        )

    @staticmethod
    def _pandas_to_json_arrow_schema(popuation_df: "pd.DataFrame") -> str:
        try:
            from pyarrow import DataType, Field, ListType, Schema, StructType
        except ImportError:
            raise QwakException(
                f"""
                Missing 'pyarrow' dependency required for fetching data from the offline store.
                {UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG}"
            """
            )

        def arrow_field_type_to_json(
            field_type: DataType,
        ) -> Union[Dict[str, str], str]:
            if isinstance(field_type, StructType):
                internal_res = {}
                for i in range(field_type.num_fields):
                    internal_field: Field = field_type.field(i)
                    internal_json_type = arrow_field_type_to_json(internal_field.type)
                    internal_res[internal_field.name] = internal_json_type

                ret_val = {"struct": internal_res}
            elif isinstance(field_type, ListType):
                value_type: DataType = field_type.value_type
                internal_json_type = arrow_field_type_to_json(value_type)
                ret_val = {"list": internal_json_type}
            else:
                ret_val = str(field_type)

            return ret_val

        arrow_schema: Schema = Schema.from_pandas(popuation_df)
        res = {}
        for i in range(len(arrow_schema.types)):
            field: Field = arrow_schema.field(i)
            json_type = arrow_field_type_to_json(field.type)
            res[field.name] = json_type

        return json.dumps(res)

    @staticmethod
    def _results_response_to_df(
        response: ProtoGetFeatureValuesResultResponse,
    ) -> "pd.DataFrame":
        if (
            response.status
            == ProtoFeatureValuesRequestStatus.FEATURE_VALUES_REQUEST_STATUS_SUCCEEDED
        ):
            dfs = [pd.read_csv(f) for f in response.link_to_files.link_to_files]
            return pd.concat(dfs)
        elif (
            response.status
            == ProtoFeatureValuesRequestStatus.FEATURE_VALUES_REQUEST_STATUS_FAILED
        ):
            failure_reason = response.failure_reason.message
            raise QwakException(
                f"Failed to get Feature Values with the following error message: {failure_reason}"
            )
        elif (
            response.status
            == ProtoFeatureValuesRequestStatus.FEATURE_VALUES_REQUEST_STATUS_CANCELLED
        ):
            raise QwakException(
                "Failed to get Feature Values, request was cancelled by another source"
            )
        else:
            raise QwakException("Got an unexpected Feature Values Status")



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class LogSource(google.protobuf.message.Message):
    """Represent a source to fetch the logs from"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_RUNTIME_FIELD_NUMBER: builtins.int
    REMOTE_BUILD_FIELD_NUMBER: builtins.int
    INFERENCE_EXECUTION_FIELD_NUMBER: builtins.int
    STREAMING_FEATURE_SET_FIELD_NUMBER: builtins.int
    STREAMING_AGGREGATION_FEATURE_SET_FIELD_NUMBER: builtins.int
    BATCH_FEATURE_SET_FIELD_NUMBER: builtins.int
    REAL_TIME_FEATURE_EXTRACTOR_FIELD_NUMBER: builtins.int
    @property
    def model_runtime(self) -> global___ModelRuntimeSource: ...
    @property
    def remote_build(self) -> global___RemoteBuildSource: ...
    @property
    def inference_execution(self) -> global___InferenceExecutionSource: ...
    @property
    def streaming_feature_set(self) -> global___StreamingFeatureSetSource: ...
    @property
    def streaming_aggregation_feature_set(self) -> global___StreamingAggregationFeatureSetSource: ...
    @property
    def batch_feature_set(self) -> global___BatchFeatureSetSource: ...
    @property
    def real_time_feature_extractor(self) -> global___RealTimeFeatureExtractorSource: ...
    def __init__(
        self,
        *,
        model_runtime: global___ModelRuntimeSource | None = ...,
        remote_build: global___RemoteBuildSource | None = ...,
        inference_execution: global___InferenceExecutionSource | None = ...,
        streaming_feature_set: global___StreamingFeatureSetSource | None = ...,
        streaming_aggregation_feature_set: global___StreamingAggregationFeatureSetSource | None = ...,
        batch_feature_set: global___BatchFeatureSetSource | None = ...,
        real_time_feature_extractor: global___RealTimeFeatureExtractorSource | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_feature_set", b"batch_feature_set", "inference_execution", b"inference_execution", "model_runtime", b"model_runtime", "real_time_feature_extractor", b"real_time_feature_extractor", "remote_build", b"remote_build", "source", b"source", "streaming_aggregation_feature_set", b"streaming_aggregation_feature_set", "streaming_feature_set", b"streaming_feature_set"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_feature_set", b"batch_feature_set", "inference_execution", b"inference_execution", "model_runtime", b"model_runtime", "real_time_feature_extractor", b"real_time_feature_extractor", "remote_build", b"remote_build", "source", b"source", "streaming_aggregation_feature_set", b"streaming_aggregation_feature_set", "streaming_feature_set", b"streaming_feature_set"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["source", b"source"]) -> typing_extensions.Literal["model_runtime", "remote_build", "inference_execution", "streaming_feature_set", "streaming_aggregation_feature_set", "batch_feature_set", "real_time_feature_extractor"] | None: ...

global___LogSource = LogSource

class RemoteBuildSource(google.protobuf.message.Message):
    """Represent a "remote build source" """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_GROUP_NAME_FIELD_NUMBER: builtins.int
    MODEL_NAME_FIELD_NUMBER: builtins.int
    MODEL_ID_FIELD_NUMBER: builtins.int
    BUILD_ID_FIELD_NUMBER: builtins.int
    PHASE_IDS_FIELD_NUMBER: builtins.int
    model_group_name: builtins.str
    """Model group name - for verifying user permissions
    This field is mandatory
    """
    model_name: builtins.str
    """This field is identical to model_id, use model_id instead"""
    model_id: builtins.str
    """Model id (human readable name)"""
    build_id: builtins.str
    """Filter by build id"""
    @property
    def phase_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The name of the build phase (not set when we want logs from all phases)"""
    def __init__(
        self,
        *,
        model_group_name: builtins.str = ...,
        model_name: builtins.str = ...,
        model_id: builtins.str = ...,
        build_id: builtins.str = ...,
        phase_ids: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["model_id", b"model_id", "model_identifier", b"model_identifier"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_id", b"build_id", "model_group_name", b"model_group_name", "model_id", b"model_id", "model_identifier", b"model_identifier", "model_name", b"model_name", "phase_ids", b"phase_ids"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["model_identifier", b"model_identifier"]) -> typing_extensions.Literal["model_id"] | None: ...

global___RemoteBuildSource = RemoteBuildSource

class ModelRuntimeSource(google.protobuf.message.Message):
    """Represent a "model runtime source" """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_GROUP_NAME_FIELD_NUMBER: builtins.int
    MODEL_NAME_FIELD_NUMBER: builtins.int
    MODEL_ID_FIELD_NUMBER: builtins.int
    DEPLOYMENT_ID_FIELD_NUMBER: builtins.int
    BUILD_ID_FIELD_NUMBER: builtins.int
    model_group_name: builtins.str
    """Model group name - for verifying user permissions
    This field is mandatory
    """
    model_name: builtins.str
    """This field is identical to model_id, use model_id instead"""
    model_id: builtins.str
    """Model id (human readable name)"""
    deployment_id: builtins.str
    """Filter by deployment id"""
    build_id: builtins.str
    """Filter by build id"""
    def __init__(
        self,
        *,
        model_group_name: builtins.str = ...,
        model_name: builtins.str = ...,
        model_id: builtins.str = ...,
        deployment_id: builtins.str = ...,
        build_id: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["build_id", b"build_id", "deployment_id", b"deployment_id", "model_id", b"model_id", "model_identifier", b"model_identifier", "search_by", b"search_by"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_id", b"build_id", "deployment_id", b"deployment_id", "model_group_name", b"model_group_name", "model_id", b"model_id", "model_identifier", b"model_identifier", "model_name", b"model_name", "search_by", b"search_by"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["model_identifier", b"model_identifier"]) -> typing_extensions.Literal["model_id"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["search_by", b"search_by"]) -> typing_extensions.Literal["deployment_id", "build_id"] | None: ...

global___ModelRuntimeSource = ModelRuntimeSource

class InferenceExecutionSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_GROUP_NAME_FIELD_NUMBER: builtins.int
    MODEL_NAME_FIELD_NUMBER: builtins.int
    MODEL_ID_FIELD_NUMBER: builtins.int
    INFERENCE_JOB_ID_FIELD_NUMBER: builtins.int
    INFERENCE_TASK_ID_FIELD_NUMBER: builtins.int
    model_group_name: builtins.str
    """Model group name - for verifying user permissions
    This field is mandatory
    """
    model_name: builtins.str
    """This field is identical to model_id, use model_id instead"""
    model_id: builtins.str
    """Model id (human readable name)"""
    inference_job_id: builtins.str
    """Filter by deployment id"""
    inference_task_id: builtins.str
    """Filter by build id"""
    def __init__(
        self,
        *,
        model_group_name: builtins.str = ...,
        model_name: builtins.str = ...,
        model_id: builtins.str = ...,
        inference_job_id: builtins.str = ...,
        inference_task_id: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["inference_job_id", b"inference_job_id", "inference_task_id", b"inference_task_id", "model_id", b"model_id", "model_identifier", b"model_identifier", "search_by", b"search_by"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["inference_job_id", b"inference_job_id", "inference_task_id", b"inference_task_id", "model_group_name", b"model_group_name", "model_id", b"model_id", "model_identifier", b"model_identifier", "model_name", b"model_name", "search_by", b"search_by"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["model_identifier", b"model_identifier"]) -> typing_extensions.Literal["model_id"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["search_by", b"search_by"]) -> typing_extensions.Literal["inference_job_id", "inference_task_id"] | None: ...

global___InferenceExecutionSource = InferenceExecutionSource

class StreamingFeatureSetSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_ID_FIELD_NUMBER: builtins.int
    FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    OFFLINE_FIELD_NUMBER: builtins.int
    ONLINE_FIELD_NUMBER: builtins.int
    feature_set_id: builtins.str
    """Filter by feature set id"""
    feature_set_name: builtins.str
    """Filter by feature set name, as well as for verifying user permissions
    This field is mandatory
    """
    feature_set_repository_name: builtins.str
    """Feature set repository name - for verifying user permissions
    This field is mandatory
    """
    @property
    def offline(self) -> global___OfflineType: ...
    @property
    def online(self) -> global___OnlineType: ...
    def __init__(
        self,
        *,
        feature_set_id: builtins.str = ...,
        feature_set_name: builtins.str = ...,
        feature_set_repository_name: builtins.str = ...,
        offline: global___OfflineType | None = ...,
        online: global___OnlineType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["offline", b"offline", "online", b"online", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_id", b"feature_set_id", "feature_set_name", b"feature_set_name", "feature_set_repository_name", b"feature_set_repository_name", "offline", b"offline", "online", b"online", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["offline", "online"] | None: ...

global___StreamingFeatureSetSource = StreamingFeatureSetSource

class BatchFeatureSetSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURESET_ID_FIELD_NUMBER: builtins.int
    FEATURESET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    FILTER_BY_FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    RUN_ID_FIELD_NUMBER: builtins.int
    featureset_id: builtins.str
    """Filter by feature set id"""
    featureset_name: builtins.str
    """Filter by feature set name, as well as for verification of user permissions
    This field is mandatory
    """
    feature_set_repository_name: builtins.str
    """Feature set repository name - for verifying user permissions
    This field is mandatory
    """
    filter_by_feature_set_name: builtins.bool
    """Optionally filter by feature set name"""
    run_id: builtins.str
    """Filter by run id"""
    def __init__(
        self,
        *,
        featureset_id: builtins.str = ...,
        featureset_name: builtins.str = ...,
        feature_set_repository_name: builtins.str = ...,
        filter_by_feature_set_name: builtins.bool = ...,
        run_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_repository_name", b"feature_set_repository_name", "featureset_id", b"featureset_id", "featureset_name", b"featureset_name", "filter_by_feature_set_name", b"filter_by_feature_set_name", "run_id", b"run_id"]) -> None: ...

global___BatchFeatureSetSource = BatchFeatureSetSource

class StreamingAggregationFeatureSetSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROW_LEVEL_FIELD_NUMBER: builtins.int
    COMPACTION_FIELD_NUMBER: builtins.int
    BACKFILL_FIELD_NUMBER: builtins.int
    @property
    def row_level(self) -> global___RowLevelType: ...
    @property
    def compaction(self) -> global___CompactionType: ...
    @property
    def backfill(self) -> global___BackfillType: ...
    def __init__(
        self,
        *,
        row_level: global___RowLevelType | None = ...,
        compaction: global___CompactionType | None = ...,
        backfill: global___BackfillType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "compaction", b"compaction", "row_level", b"row_level", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "compaction", b"compaction", "row_level", b"row_level", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["row_level", "compaction", "backfill"] | None: ...

global___StreamingAggregationFeatureSetSource = StreamingAggregationFeatureSetSource

class OnlineType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_ID_FIELD_NUMBER: builtins.int
    FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    feature_set_id: builtins.str
    """Filter by feature set id"""
    feature_set_name: builtins.str
    """Filter by feature set name, as well as for verification of user permissions
    This field is mandatory
    """
    feature_set_repository_name: builtins.str
    """Feature set repository name - for verifying user permissions
    This field is mandatory
    """
    def __init__(
        self,
        *,
        feature_set_id: builtins.str = ...,
        feature_set_name: builtins.str = ...,
        feature_set_repository_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_id", b"feature_set_id", "feature_set_name", b"feature_set_name", "feature_set_repository_name", b"feature_set_repository_name"]) -> None: ...

global___OnlineType = OnlineType

class OfflineType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_ID_FIELD_NUMBER: builtins.int
    FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    RUN_ID_FIELD_NUMBER: builtins.int
    feature_set_id: builtins.str
    """Filter by feature set id"""
    feature_set_name: builtins.str
    """Filter by feature set name, as well as for verification of user permissions
    This field is mandatory
    """
    feature_set_repository_name: builtins.str
    """Feature set repository name - for verifying user permissions
    This field is mandatory
    """
    run_id: builtins.str
    """Filter by run id"""
    def __init__(
        self,
        *,
        feature_set_id: builtins.str = ...,
        feature_set_name: builtins.str = ...,
        feature_set_repository_name: builtins.str = ...,
        run_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_id", b"feature_set_id", "feature_set_name", b"feature_set_name", "feature_set_repository_name", b"feature_set_repository_name", "run_id", b"run_id"]) -> None: ...

global___OfflineType = OfflineType

class RowLevelType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_ID_FIELD_NUMBER: builtins.int
    FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    feature_set_id: builtins.str
    """Filter by feature set id"""
    feature_set_name: builtins.str
    """Filter by feature set name
    This field is mandatory
    """
    feature_set_repository_name: builtins.str
    """Feature set repository name - for verifying user permissions
    This field is mandatory
    """
    def __init__(
        self,
        *,
        feature_set_id: builtins.str = ...,
        feature_set_name: builtins.str = ...,
        feature_set_repository_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_id", b"feature_set_id", "feature_set_name", b"feature_set_name", "feature_set_repository_name", b"feature_set_repository_name"]) -> None: ...

global___RowLevelType = RowLevelType

class CompactionType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_ID_FIELD_NUMBER: builtins.int
    FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    RUN_ID_FIELD_NUMBER: builtins.int
    feature_set_id: builtins.str
    """Filter by feature set id"""
    feature_set_name: builtins.str
    """Filter by feature set name, as well as for verification of user permissions
    This field is mandatory
    """
    feature_set_repository_name: builtins.str
    """Feature set repository name - for verifying user permissions
    This field is mandatory
    """
    run_id: builtins.str
    """Filter by run id"""
    def __init__(
        self,
        *,
        feature_set_id: builtins.str = ...,
        feature_set_name: builtins.str = ...,
        feature_set_repository_name: builtins.str = ...,
        run_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_id", b"feature_set_id", "feature_set_name", b"feature_set_name", "feature_set_repository_name", b"feature_set_repository_name", "run_id", b"run_id"]) -> None: ...

global___CompactionType = CompactionType

class BackfillType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_ID_FIELD_NUMBER: builtins.int
    FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_REPOSITORY_NAME_FIELD_NUMBER: builtins.int
    feature_set_id: builtins.str
    """Filter by feature set id"""
    feature_set_name: builtins.str
    """Filter by feature set name, as well as for verification of user permissions
    This field is mandatory
    """
    feature_set_repository_name: builtins.str
    """Feature set repository name - for verifying user permissions
    This field is mandatory
    """
    def __init__(
        self,
        *,
        feature_set_id: builtins.str = ...,
        feature_set_name: builtins.str = ...,
        feature_set_repository_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_set_id", b"feature_set_id", "feature_set_name", b"feature_set_name", "feature_set_repository_name", b"feature_set_repository_name"]) -> None: ...

global___BackfillType = BackfillType

class RealTimeFeatureExtractorSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REAL_TIME_FEATURE_EXTRACTOR_ID_FIELD_NUMBER: builtins.int
    real_time_feature_extractor_id: builtins.str
    """Filter by the real-time feature extractor id"""
    def __init__(
        self,
        *,
        real_time_feature_extractor_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["real_time_feature_extractor_id", b"real_time_feature_extractor_id"]) -> None: ...

global___RealTimeFeatureExtractorSource = RealTimeFeatureExtractorSource



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/features/feature_set_state_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.feature_store.features import feature_set_state_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__state__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;qwak/feature_store/features/feature_set_state_service.proto\x12\x1bqwak.feature.store.features\x1a\x33qwak/feature_store/features/feature_set_state.proto\"a\n\x1fGetFeatureSetStateByIdV1Request\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12\x0e\n\x06run_id\x18\x02 \x01(\x03\x12\x16\n\x0e\x65nvironment_id\x18\x03 \x01(\t\"k\n GetFeatureSetStateByIdV1Response\x12G\n\x11\x66\x65\x61ture_set_state\x18\x01 \x01(\x0b\x32,.qwak.feature.store.features.FeatureSetState\"e\n!GetFeatureSetStateByNameV1Request\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\x12\x0e\n\x06run_id\x18\x02 \x01(\x03\x12\x16\n\x0e\x65nvironment_id\x18\x03 \x01(\t\"m\n\"GetFeatureSetStateByNameV1Response\x12G\n\x11\x66\x65\x61ture_set_state\x18\x01 \x01(\x0b\x32,.qwak.feature.store.features.FeatureSetState\"\"\n GetFeatureSetStatesListV1Request\"l\n!GetFeatureSetStatesListV1Response\x12G\n\x11\x66\x65\x61ture_set_state\x18\x01 \x03(\x0b\x32,.qwak.feature.store.features.FeatureSetState\"\x81\x01\n\x1eUpdateFeatureSetStateV1Request\x12G\n\x11\x66\x65\x61ture_set_state\x18\x01 \x03(\x0b\x32,.qwak.feature.store.features.FeatureSetState\x12\x16\n\x0e\x65nvironment_id\x18\x02 \x01(\t\"!\n\x1fUpdateFeatureSetStateV1Response\"!\n\x1f\x44\x65leteFeatureSetStateV1Response\"8\n\x1e\x44\x65leteFeatureSetStateV1Request\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t2\x93\x06\n\x16\x46\x65\x61tureSetStateService\x12\x95\x01\n\x16GetFeatureSetStateById\x12<.qwak.feature.store.features.GetFeatureSetStateByIdV1Request\x1a=.qwak.feature.store.features.GetFeatureSetStateByIdV1Response\x12\x9b\x01\n\x18GetFeatureSetStateByName\x12>.qwak.feature.store.features.GetFeatureSetStateByNameV1Request\x1a?.qwak.feature.store.features.GetFeatureSetStateByNameV1Response\x12\x98\x01\n\x17GetFeatureSetStatesList\x12=.qwak.feature.store.features.GetFeatureSetStatesListV1Request\x1a>.qwak.feature.store.features.GetFeatureSetStatesListV1Response\x12\x92\x01\n\x15UpdateFeatureSetState\x12;.qwak.feature.store.features.UpdateFeatureSetStateV1Request\x1a<.qwak.feature.store.features.UpdateFeatureSetStateV1Response\x12\x92\x01\n\x15\x44\x65leteFeatureSetState\x12;.qwak.feature.store.features.DeleteFeatureSetStateV1Request\x1a<.qwak.feature.store.features.DeleteFeatureSetStateV1ResponseB]\n(com.qwak.ai.feature.store.management.apiP\x01Z/qwak/featurestore/features;featurestorefeaturesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.feature_store.features.feature_set_state_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n(com.qwak.ai.feature.store.management.apiP\001Z/qwak/featurestore/features;featurestorefeatures'
  _globals['_GETFEATURESETSTATEBYIDV1REQUEST']._serialized_start=145
  _globals['_GETFEATURESETSTATEBYIDV1REQUEST']._serialized_end=242
  _globals['_GETFEATURESETSTATEBYIDV1RESPONSE']._serialized_start=244
  _globals['_GETFEATURESETSTATEBYIDV1RESPONSE']._serialized_end=351
  _globals['_GETFEATURESETSTATEBYNAMEV1REQUEST']._serialized_start=353
  _globals['_GETFEATURESETSTATEBYNAMEV1REQUEST']._serialized_end=454
  _globals['_GETFEATURESETSTATEBYNAMEV1RESPONSE']._serialized_start=456
  _globals['_GETFEATURESETSTATEBYNAMEV1RESPONSE']._serialized_end=565
  _globals['_GETFEATURESETSTATESLISTV1REQUEST']._serialized_start=567
  _globals['_GETFEATURESETSTATESLISTV1REQUEST']._serialized_end=601
  _globals['_GETFEATURESETSTATESLISTV1RESPONSE']._serialized_start=603
  _globals['_GETFEATURESETSTATESLISTV1RESPONSE']._serialized_end=711
  _globals['_UPDATEFEATURESETSTATEV1REQUEST']._serialized_start=714
  _globals['_UPDATEFEATURESETSTATEV1REQUEST']._serialized_end=843
  _globals['_UPDATEFEATURESETSTATEV1RESPONSE']._serialized_start=845
  _globals['_UPDATEFEATURESETSTATEV1RESPONSE']._serialized_end=878
  _globals['_DELETEFEATURESETSTATEV1RESPONSE']._serialized_start=880
  _globals['_DELETEFEATURESETSTATEV1RESPONSE']._serialized_end=913
  _globals['_DELETEFEATURESETSTATEV1REQUEST']._serialized_start=915
  _globals['_DELETEFEATURESETSTATEV1REQUEST']._serialized_end=971
  _globals['_FEATURESETSTATESERVICE']._serialized_start=974
  _globals['_FEATURESETSTATESERVICE']._serialized_end=1761
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/execution/v1/state/featureset_state.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.qwak/execution/v1/state/featureset_state.proto\x12%qwak.feature.store.execution.v1.state\"A\n\x0f\x46\x65\x61turesetState\x12\x15\n\rfeatureset_id\x18\x01 \x01(\t\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x02 \x01(\tB`\n+com.qwak.ai.features.execution.api.v1.stateP\x01Z/qwak/execution_management/state;execution_stateb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.execution.v1.state.featureset_state_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n+com.qwak.ai.features.execution.api.v1.stateP\001Z/qwak/execution_management/state;execution_state'
  _globals['_FEATURESETSTATE']._serialized_start=89
  _globals['_FEATURESETSTATE']._serialized_end=154
# @@protoc_insertion_point(module_scope)



import grpc
from _qwak_proto.qwak.builds.build_url_pb2 import (
    BuildVersioningTagsDefinition,
    BuildVersioningTagsProperties,
    BuildVersioningTagsType,
)
from _qwak_proto.qwak.builds.build_pb2 import (
    BaseDockerImageType,
)
from _qwak_proto.qwak.builds.builds_orchestrator_service_pb2 import (
    BuildModelResponse,
    CancelBuildModelResponse,
    CreateDataTableResponse,
    GetBuildVersioningDownloadURLResponse,
    GetBuildVersioningUploadURLResponse,
    ListBuildVersioningTagsResponse,
    GetBaseDockerImageNameResponse,
)
from _qwak_proto.qwak.builds.builds_orchestrator_service_pb2_grpc import (
    BuildsOrchestratorServiceServicer,
)
from qwak_services_mock.mocks.utils.exception_handlers import raise_internal_grpc_error


class BuildOrchestratorServiceApiMock(BuildsOrchestratorServiceServicer):
    UPLOAD_URL_FORMAT = "/{build_id}/upload_urls/file.zip"
    DOWNLOAD_URL_FORMAT = "/{build_id}/download_urls/file.zip"

    def __init__(self):
        super(BuildOrchestratorServiceApiMock, self).__init__()

    def BuildModel(self, request, context):
        """Build a serving model image"""
        try:
            return BuildModelResponse()
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def CancelBuildModel(self, request, context):
        """Cancel an ongoing build"""
        try:
            return CancelBuildModelResponse()
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def CreateUploadURL(self, request, context):
        """Request the service for a path ready for upload in purpose of uploading local model code available to the service
        and the download link corresponding
        """
        context.set_code(grpc.StatusCode.INTERNAL)
        context.set_details("Deprecated")
        raise DeprecationWarning("This api is deprecated and should not be used")

    def GetBuildVersioningUploadURL(self, request, context):
        """Request the service for a path ready for upload in purpose of uploading local model code or data"""
        try:
            return GetBuildVersioningUploadURLResponse(
                upload_url=BuildOrchestratorServiceApiMock.UPLOAD_URL_FORMAT.format(
                    build_id=request.params.build_id
                ),
            )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetBuildVersioningDownloadURL(self, request, context):
        """Request the service for a path ready for download the artifact"""
        try:
            if request.params.build_id == "not_found":
                context.set_code(grpc.StatusCode.NOT_FOUND)
                context.set_details(str("key not found"))
                return GetBuildVersioningDownloadURLResponse()
            return GetBuildVersioningDownloadURLResponse(
                download_url=BuildOrchestratorServiceApiMock.DOWNLOAD_URL_FORMAT.format(
                    build_id=request.params.build_id
                ),
                file_size=1000,
            )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def ListBuildVersioningTags(self, request, context):
        """Request the service for a list of paths ready for download the artifact"""
        try:
            return ListBuildVersioningTagsResponse(
                build_versioning_tags_properties=[
                    self.__map_versioning_tag_to_property(
                        build_id=request.build_versioning_tags.build_id,
                        versioning_tag=tag,
                    )
                    for tag in request.build_versioning_tags.tags_definition
                ]
            )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def CreateDataTable(self, request, context):
        """Create data table for build."""
        try:
            return CreateDataTableResponse()
        except Exception as e:
            raise_internal_grpc_error(context, e)

    def GetBaseDockerImageName(self, request, context):
        try:
            if request.base_docker_image_type == BaseDockerImageType.GPU:
                return GetBaseDockerImageNameResponse(
                    base_docker_image_name="mocked_gpu_image"
                )
            elif request.base_docker_image_type == BaseDockerImageType.CPU:
                return GetBaseDockerImageNameResponse(
                    base_docker_image_name="mocked_cpu_image"
                )
            else:
                raise ValueError(
                    f"Unknown base docker image type: {request.base_docker_image_type}"
                )
        except Exception as e:
            raise_internal_grpc_error(context, e)

    @staticmethod
    def __map_versioning_tag_to_property(
        build_id: str, versioning_tag: BuildVersioningTagsDefinition
    ) -> BuildVersioningTagsProperties:
        return BuildVersioningTagsProperties(
            tag=versioning_tag.tags[0],
            download_url=BuildOrchestratorServiceApiMock.DOWNLOAD_URL_FORMAT.format(
                build_id=build_id
            ),
            file_size=1000,
            tag_type=BuildVersioningTagsType.FILE_TAG_TYPE,
        )



import pathlib

from qwak.clients.feature_store.execution_management_client import (
    ExecutionManagementClient,
)
from qwak.feature_store.feature_sets.streaming_backfill import StreamingBackfill


class StreamingAggregationBackfill:
    def __init__(
        self,
        streaming_backfill: StreamingBackfill,
        source_definition_path: pathlib.Path,
    ):
        """
        Initialize the streaming aggregation backfill executor.

        Args:
            streaming_backfill (StreamingBackfill): Specification containing the
                              featureset name, time range, data sources, and transformation
            source_definition_path (Path): Path to the Python file containing the backfill
                                   definition. Required for locating UDF artifacts.
        """
        self._streaming_backfill = streaming_backfill
        self._source_definition_path = source_definition_path

    def trigger(self) -> str:
        """
        Triggers the streaming aggregation backfill execution.

        Converts the backfill specification to proto format and sends it to
        the execution manager to start the backfill job.

        Returns:
            str: The execution ID for tracking the backfill job status

        Raises:
            QwakException: If the execution manager request fails
        """
        backfill_proto = self._streaming_backfill._to_proto(
            str(self._source_definition_path)
        )

        execution_client = ExecutionManagementClient()
        response = execution_client.trigger_streaming_aggregation_backfill(
            backfill_proto
        )
        return response.execution_id



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.models import models_pb2 as qwak_dot_models_dot_models__pb2


class ModelsManagementServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateModel = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/CreateModel',
                request_serializer=qwak_dot_models_dot_models__pb2.CreateModelRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.CreateModelResponse.FromString,
                )
        self.ListModels = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/ListModels',
                request_serializer=qwak_dot_models_dot_models__pb2.ListModelsRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.ListModelsResponse.FromString,
                )
        self.ListModelsPaginated = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/ListModelsPaginated',
                request_serializer=qwak_dot_models_dot_models__pb2.ListModelsPaginatedRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.ListModelsPaginatedResponse.FromString,
                )
        self.GetModel = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/GetModel',
                request_serializer=qwak_dot_models_dot_models__pb2.GetModelRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.GetModelResponse.FromString,
                )
        self.ListModelsByModelIds = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/ListModelsByModelIds',
                request_serializer=qwak_dot_models_dot_models__pb2.ListModelsByModelIdsRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.ListModelsResponse.FromString,
                )
        self.DeleteModel = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/DeleteModel',
                request_serializer=qwak_dot_models_dot_models__pb2.DeleteModelRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.DeleteModelResponse.FromString,
                )
        self.UpdateModel = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/UpdateModel',
                request_serializer=qwak_dot_models_dot_models__pb2.UpdateModelRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.UpdateModelResponse.FromString,
                )
        self.CreateBranch = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/CreateBranch',
                request_serializer=qwak_dot_models_dot_models__pb2.CreateBranchRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.CreateBranchResponse.FromString,
                )
        self.DeleteBranch = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/DeleteBranch',
                request_serializer=qwak_dot_models_dot_models__pb2.DeleteBranchRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.DeleteBranchResponse.FromString,
                )
        self.GetDeploymentDetails = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/GetDeploymentDetails',
                request_serializer=qwak_dot_models_dot_models__pb2.GetDeploymentDetailsRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.GetDeploymentDetailsResponse.FromString,
                )
        self.IsModelExists = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/IsModelExists',
                request_serializer=qwak_dot_models_dot_models__pb2.IsModelExistsRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.IsModelExistsResponse.FromString,
                )
        self.ListModelsMetadata = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/ListModelsMetadata',
                request_serializer=qwak_dot_models_dot_models__pb2.ListModelsMetadataRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.ListModelsMetadataResponse.FromString,
                )
        self.GetModelMetadata = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/GetModelMetadata',
                request_serializer=qwak_dot_models_dot_models__pb2.GetModelMetadataRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.GetModelMetadataResponse.FromString,
                )
        self.CheckModelCreationAllowed = channel.unary_unary(
                '/qwak.models.management.ModelsManagementService/CheckModelCreationAllowed',
                request_serializer=qwak_dot_models_dot_models__pb2.CheckModelCreationAllowedRequest.SerializeToString,
                response_deserializer=qwak_dot_models_dot_models__pb2.CheckModelCreationAllowedResponse.FromString,
                )


class ModelsManagementServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateModel(self, request, context):
        """Create model
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListModels(self, request, context):
        """List all models
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListModelsPaginated(self, request, context):
        """Retrieves a paginated list of models based on the provided filters and pagination parameters.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetModel(self, request, context):
        """Get model
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListModelsByModelIds(self, request, context):
        """Get models by model IDs and model UUIDs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteModel(self, request, context):
        """Delete model
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateModel(self, request, context):
        """Update model
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateBranch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteBranch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDeploymentDetails(self, request, context):
        """Get model's deployment details
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def IsModelExists(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListModelsMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetModelMetadata(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CheckModelCreationAllowed(self, request, context):
        """Check if model creation allowed for the provided information: model group name, jfrog project key and model id
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ModelsManagementServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateModel': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateModel,
                    request_deserializer=qwak_dot_models_dot_models__pb2.CreateModelRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.CreateModelResponse.SerializeToString,
            ),
            'ListModels': grpc.unary_unary_rpc_method_handler(
                    servicer.ListModels,
                    request_deserializer=qwak_dot_models_dot_models__pb2.ListModelsRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.ListModelsResponse.SerializeToString,
            ),
            'ListModelsPaginated': grpc.unary_unary_rpc_method_handler(
                    servicer.ListModelsPaginated,
                    request_deserializer=qwak_dot_models_dot_models__pb2.ListModelsPaginatedRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.ListModelsPaginatedResponse.SerializeToString,
            ),
            'GetModel': grpc.unary_unary_rpc_method_handler(
                    servicer.GetModel,
                    request_deserializer=qwak_dot_models_dot_models__pb2.GetModelRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.GetModelResponse.SerializeToString,
            ),
            'ListModelsByModelIds': grpc.unary_unary_rpc_method_handler(
                    servicer.ListModelsByModelIds,
                    request_deserializer=qwak_dot_models_dot_models__pb2.ListModelsByModelIdsRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.ListModelsResponse.SerializeToString,
            ),
            'DeleteModel': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteModel,
                    request_deserializer=qwak_dot_models_dot_models__pb2.DeleteModelRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.DeleteModelResponse.SerializeToString,
            ),
            'UpdateModel': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateModel,
                    request_deserializer=qwak_dot_models_dot_models__pb2.UpdateModelRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.UpdateModelResponse.SerializeToString,
            ),
            'CreateBranch': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateBranch,
                    request_deserializer=qwak_dot_models_dot_models__pb2.CreateBranchRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.CreateBranchResponse.SerializeToString,
            ),
            'DeleteBranch': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteBranch,
                    request_deserializer=qwak_dot_models_dot_models__pb2.DeleteBranchRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.DeleteBranchResponse.SerializeToString,
            ),
            'GetDeploymentDetails': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDeploymentDetails,
                    request_deserializer=qwak_dot_models_dot_models__pb2.GetDeploymentDetailsRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.GetDeploymentDetailsResponse.SerializeToString,
            ),
            'IsModelExists': grpc.unary_unary_rpc_method_handler(
                    servicer.IsModelExists,
                    request_deserializer=qwak_dot_models_dot_models__pb2.IsModelExistsRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.IsModelExistsResponse.SerializeToString,
            ),
            'ListModelsMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.ListModelsMetadata,
                    request_deserializer=qwak_dot_models_dot_models__pb2.ListModelsMetadataRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.ListModelsMetadataResponse.SerializeToString,
            ),
            'GetModelMetadata': grpc.unary_unary_rpc_method_handler(
                    servicer.GetModelMetadata,
                    request_deserializer=qwak_dot_models_dot_models__pb2.GetModelMetadataRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.GetModelMetadataResponse.SerializeToString,
            ),
            'CheckModelCreationAllowed': grpc.unary_unary_rpc_method_handler(
                    servicer.CheckModelCreationAllowed,
                    request_deserializer=qwak_dot_models_dot_models__pb2.CheckModelCreationAllowedRequest.FromString,
                    response_serializer=qwak_dot_models_dot_models__pb2.CheckModelCreationAllowedResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.models.management.ModelsManagementService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ModelsManagementService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateModel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/CreateModel',
            qwak_dot_models_dot_models__pb2.CreateModelRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.CreateModelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListModels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/ListModels',
            qwak_dot_models_dot_models__pb2.ListModelsRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.ListModelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListModelsPaginated(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/ListModelsPaginated',
            qwak_dot_models_dot_models__pb2.ListModelsPaginatedRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.ListModelsPaginatedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetModel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/GetModel',
            qwak_dot_models_dot_models__pb2.GetModelRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.GetModelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListModelsByModelIds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/ListModelsByModelIds',
            qwak_dot_models_dot_models__pb2.ListModelsByModelIdsRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.ListModelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteModel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/DeleteModel',
            qwak_dot_models_dot_models__pb2.DeleteModelRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.DeleteModelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateModel(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/UpdateModel',
            qwak_dot_models_dot_models__pb2.UpdateModelRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.UpdateModelResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateBranch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/CreateBranch',
            qwak_dot_models_dot_models__pb2.CreateBranchRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.CreateBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteBranch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/DeleteBranch',
            qwak_dot_models_dot_models__pb2.DeleteBranchRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.DeleteBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDeploymentDetails(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/GetDeploymentDetails',
            qwak_dot_models_dot_models__pb2.GetDeploymentDetailsRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.GetDeploymentDetailsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def IsModelExists(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/IsModelExists',
            qwak_dot_models_dot_models__pb2.IsModelExistsRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.IsModelExistsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListModelsMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/ListModelsMetadata',
            qwak_dot_models_dot_models__pb2.ListModelsMetadataRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.ListModelsMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetModelMetadata(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/GetModelMetadata',
            qwak_dot_models_dot_models__pb2.GetModelMetadataRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.GetModelMetadataResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CheckModelCreationAllowed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.models.management.ModelsManagementService/CheckModelCreationAllowed',
            qwak_dot_models_dot_models__pb2.CheckModelCreationAllowedRequest.SerializeToString,
            qwak_dot_models_dot_models__pb2.CheckModelCreationAllowedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
